#!/bin/sh
. /usr/share/libubox/jshn.sh

SECTION=$1

ENABLE=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".enable)
TIMEOUT=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".time_out)
RETRIES=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".retry)
HOST=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".host)
HOST2=$(uci -q get ping_reboot."$SECTION".host2)
ACTION=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".action)
CURRENT_TRY=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".current_try)
PACKET_SIZE=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".packet_size)
MODEM_EXEC=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".modem)
TYPE=$(uci -c /var/ -q get ping_reboot_tmp."$SECTION".type)

FILE="/tmp/wget_check_file"

debug() {
    logger -t ping_reboot.sh "$1"
}

get_modem() {
    local modem modems id builtin primary
    local primary_modem=""
    local builtin_modem=""
    json_init
    json_load "$(cat /etc/board.json)"
    json_get_keys modems modems
    json_select modems

    for modem in $modems; do
        json_select "$modem"
        json_get_vars builtin primary id
        if [ "$builtin" -a "$id" ]; then
            builtin_modem=$id
        fi
        if [ "$primary" -a "$id" ]; then
            primary_modem=$id
            break
        fi
        json_select ..
    done

    if [ "$primary_modem" = "" ]; then
        if [ "$builtin_modem" = "" ]; then
            json_load "$(/bin/ubus call gsmd get_modems)"
            json_get_keys modems modems
            json_select modems

            for modem in $modems; do
                json_select "$modem"
                json_get_vars id
                primary_modem=$id
                break
            done
        else
            primary_modem=$builtin_modem
        fi
    fi
    uci -c /var/ -q set ping_reboot_tmp."$SECTION".modem="$primary_modem"
    uci -c /var/ commit ping_reboot_tmp
    MODEM_EXEC=$primary_modem
}

exec_action() {
    uci -c /var/ -c /var/ -q set ping_reboot_tmp."$SECTION".current_try=0
    uci -c /var/ commit ping_reboot_tmp
    case "$ACTION" in
        "1")
            debug "Rebooting router after $RETRIES unsuccessful tries"
            /etc/init.d/modbus-master stop
            /etc/init.d/modbus-to-iec104 stop
            sleep 5
            reboot 
            ;;
        "2")
            debug "Restarting modem after $RETRIES unsuccessful tries"
            /bin/ubus call gsmd reinit_modems "{\"id\":\"$MODEM_EXEC\"}"
            ;;
        "3")
            ;;
    esac
}

if [ "$ENABLE" == "1" ]; then
    if [ "$MODEM_EXEC" = "" ]; then
        get_modem
    fi
    CURRENT_TRY=$((CURRENT_TRY + 1))
    uci -c /var/ -q set ping_reboot_tmp."$SECTION".current_try=$CURRENT_TRY
    uci -c /var/ commit ping_reboot_tmp

    if [ "$TYPE" = "wget" ]; then
        wget -q -T $TIMEOUT $HOST -O $FILE &> /dev/null
        if [ ! -s $FILE ];then
            debug "Can't wget URL."
            if [ $CURRENT_TRY -ge $RETRIES ]; then
                exec_action "-g"
            else
                debug "Will be retrying wget"
            fi
        else
            uci -c /var/ -q set ping_reboot_tmp."$SECTION".current_try=0
            uci -c /var/ commit ping_reboot_tmp
            debug "Wget URL successful."
        fi
        rm $FILE &> /dev/null
    else
        result=$(ping -W $TIMEOUT -s $PACKET_SIZE -q -c 1 $HOST &> /dev/null; echo $?)
        if [ "$result" != "0" ]; then
             result=$(ping -W $TIMEOUT -s $PACKET_SIZE -q -c 1 $HOST2 &> /dev/null; echo $?)
        fi
        if [ "$result" != "0" ]; then
            debug "Can't ping URL."
            if [ $CURRENT_TRY -ge $RETRIES ]; then
                exec_action "-p"
            else
                debug "Will be retrying ping"
            fi
        else
            uci -c /var/ -q set ping_reboot_tmp."$SECTION".current_try=0
            uci -c /var/ commit ping_reboot_tmp
            debug "Ping URL successful."
        fi
    fi
fi